@@ -37,6 +37,8 @@ from paleomix.common.console import \
     print_info, \
     print_warn
 
+from paleomix.common.formats.fasta import FASTA
+
 from paleomix.pipeline import \
     Pypeline
 
@@ -122,7 +124,7 @@ def build_admixture_nodes(config, data, root, plink):
 
         admix_root = os.path.join(root, "results", "admixture")
         report_root = os.path.join(root, "figures", "admixture")
-        for k_groups in (2, 3):
+        for k_groups in sorted(data.groups):
             replicates = []
 
             input_file = os.path.join(plink["root"], postfix + ".bed")
@@ -132,7 +134,7 @@ def build_admixture_nodes(config, data, root, plink):
                 node = nuclear.AdmixtureNode(input_file=input_file,
                                              output_root=output_root,
                                              k_groups=k_groups,
-                                             samples=data.samples,
+                                             groups=data.groups[k_groups],
                                              dependencies=(bed_node,))
 
                 replicates.append(node)
@@ -228,10 +230,11 @@ def build_pca_nodes(config, data, root, plink):
     return nodes
 
 
-def build_coverage_nodes(data, root, nuc_bam, dependencies=()):
+def build_coverage_nodes(contigs, mapping, root, nuc_bam, dependencies=()):
     output_prefix = os.path.join(root, 'figures', 'coverage', 'coverage')
 
-    return nuclear.PlotCoverageNode(contigs=data.contigs,
+    return nuclear.PlotCoverageNode(contigs=contigs,
+                                    mapping=mapping,
                                     input_file=nuc_bam,
                                     output_prefix=output_prefix,
                                     dependencies=dependencies),
@@ -278,6 +281,8 @@ def build_pipeline(config, root, nuc_bam, mito_bam, cache):
                                            output_file=sample_tbl)
 
     if nuc_bam is not None:
+        nuc_bam, nuc_bam_info = nuc_bam["Path"], nuc_bam["Info"]
+
         # When not sampling, BuildTPED relies on indexed access to ease
         # processing of one chromosome at a time. The index is further required
         # for idxstats used by the PlotCoverageNode.
@@ -292,8 +297,11 @@ def build_pipeline(config, root, nuc_bam, mito_bam, cache):
                                            plink))
 
         if not config.admixture_only:
-            nodes.extend(build_coverage_nodes(config.database,
-                                              root, nuc_bam, (index,)))
+            nodes.extend(build_coverage_nodes(contigs=config.database.contigs,
+                                              mapping=nuc_bam_info.nuclear_contigs,
+                                              root=root,
+                                              nuc_bam=nuc_bam,
+                                              dependencies=(index,)))
             nodes.extend(build_pca_nodes(config, config.database,
                                          root, plink))
             nodes.extend(build_treemix_nodes(config, config.database,
@@ -384,8 +392,6 @@ def setup_mito_mapping(config):
 
             info = config.database.samples.get(record.name)
             if info is not None:
-                mkfile.write("    # Group: %s\n"
-                             % (info.get('Group(3)', 'NA'),))
                 mkfile.write("    # Species: %s\n"
                              % (info.get('Species', 'NA'),))
                 mkfile.write("    # Sex: %s\n"
@@ -401,6 +407,11 @@ def setup_mito_mapping(config):
                                        "%s.fasta" % (record.name,))
 
             with open(fasta_fpath, "w") as fasta_handle:
+                record = FASTA(
+                    name=record.name,
+                    meta=None,
+                    sequence=record.sequence.replace('-', ''))
+
                 fasta_handle.write(str(record))
                 fasta_handle.write("\n")
 
